#include <cuda_runtime.h>

#include <cstring>
#include <iostream>
#include <fstream>
#include <google/protobuf/message.h>
#include <google/protobuf/descriptor.h>

#include "caffe/caffe.hpp"

using namespace caffe;
namespace gp = google::protobuf;

int main(int argc, char** argv) {
  ::google::InitGoogleLogging(argv[0]);
  if (argc < 2) {
    LOG(ERROR) << "Usage: net_snapshot_to_binary input_binary_snapshot_file output_matlab_readable_binary";
    return 0;
  }

  NetParameter net_param;
  LOG(INFO) << "Reading from " << argv[1];
  ReadProtoFromBinaryFile(argv[1], &net_param);
  int num_layers = net_param.layers_size();
  for (int i = 0; i < num_layers; ++i) {
    const LayerParameter& curr_layer = net_param.layers(i);
    const string& curr_layer_name = curr_layer.name();
    LOG(INFO) << "Layer number " << i << ": " << curr_layer_name << ". Has " << curr_layer.blobs_size() << " blobs.";
    for (int j=0; j < curr_layer.blobs_size(); ++j) {
      const BlobProto& curr_blob = curr_layer.blobs(j);
      const gp::Reflection* paReflection = curr_blob.GetReflection();
      LOG(INFO) << "  Blob " << j;
      for (int fn = 0 ; fn < 100; fn++) {
        const gp::FieldDescriptor* paFieldDescriptor = curr_blob.GetDescriptor()->FindFieldByNumber(fn);
        if (paFieldDescriptor!= NULL) {
          int fval = paReflection->GetInt32( curr_blob, paFieldDescriptor);
          LOG(INFO) << "field " << fn << ": " << fval;
        }
      }
    }
  }
 
  return 0;
}

